SCRIPT_NAME=aout
TEXT_START_ADDR=0x1020
case ${LD_FLAG} in
    n|N)	TEXT_START_ADDR=0x1000 ;;
esac
OUTPUT_FORMAT="a.out-arm-netbsd"
TARGET_PAGE_SIZE=0x1000
ARCH=arm
EXECUTABLE_SYMBOLS='__DYNAMIC = 0;'
