[^:]*: Assembler messages:
[^:]*:2: Warning: swp{b} use is obsoleted for ARMv8 and later
[^:]*:6: Warning: swp{b} use is deprecated for ARMv6 and ARMv7
[^:]*:8: Warning: swp{b} use is deprecated for ARMv6 and ARMv7
