	.globl text_symbol1
	.globl text_symbol2
	.globl text_symbol3
	.text
text_symbol1:
	.long 0
	.long 0
	.long 0
	.size text_symbol1, . - text_symbol1
text_symbol2:
	.long 0
	.long 0
	.size text_symbol2, . - text_symbol2
text_symbol3:
	.long 0
	.size text_symbol3, . - text_symbol3
