.*:     file format elf64-(little|big)aarch64

Contents of the .eh_frame section:

0+0000 0+0044 0+0000 CIE
  Version:               1
  Augmentation:          "zR"
  Code alignment factor: 2
  Data alignment factor: -4
  Return address column: 30
  Augmentation data:     1b

  DW_CFA_def_cfa: r31 ofs 0
  DW_CFA_def_cfa: r0 ofs 0
  DW_CFA_offset_extended_sf: r19 at cfa\+16
  DW_CFA_offset_extended_sf: r20 at cfa\+16
  DW_CFA_offset_extended_sf: r21 at cfa\+16
  DW_CFA_offset_extended_sf: r22 at cfa\+16
  DW_CFA_offset_extended_sf: r23 at cfa\+24
  DW_CFA_offset_extended_sf: r24 at cfa\+24
  DW_CFA_offset_extended_sf: r25 at cfa\+24
  DW_CFA_offset_extended_sf: r26 at cfa\+24
  DW_CFA_offset_extended_sf: r27 at cfa\+24
  DW_CFA_offset_extended_sf: r28 at cfa\+32
  DW_CFA_offset_extended_sf: r29 at cfa\+32
  DW_CFA_offset_extended_sf: r30 at cfa\+36
  DW_CFA_offset_extended_sf: r9 at cfa\+8
  DW_CFA_offset_extended_sf: r11 at cfa\+8
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop

0+0048 0+0014 0+004c FDE cie=0+0000 pc=f+fffff80..f+fffffc0
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop

0+0060 0+0044 0+0000 CIE
  Version:               1
  Augmentation:          "zR"
  Code alignment factor: 2
  Data alignment factor: -4
  Return address column: 30
  Augmentation data:     1b

  DW_CFA_def_cfa: r31 ofs 0
  DW_CFA_def_cfa: r0 ofs 0
  DW_CFA_offset_extended_sf: r19 at cfa\+16
  DW_CFA_offset_extended_sf: r20 at cfa\+16
  DW_CFA_offset_extended_sf: r21 at cfa\+16
  DW_CFA_offset_extended_sf: r22 at cfa\+16
  DW_CFA_offset_extended_sf: r23 at cfa\+24
  DW_CFA_offset_extended_sf: r24 at cfa\+24
  DW_CFA_offset_extended_sf: r25 at cfa\+24
  DW_CFA_offset_extended_sf: r26 at cfa\+24
  DW_CFA_offset_extended_sf: r27 at cfa\+24
  DW_CFA_offset_extended_sf: r28 at cfa\+32
  DW_CFA_offset_extended_sf: r29 at cfa\+32
  DW_CFA_offset_extended_sf: r30 at cfa\+36
  DW_CFA_offset_extended_sf: r9 at cfa\+8
  DW_CFA_offset_extended_sf: r10 at cfa\+8
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop

0+00a8 0+0014 0+004c FDE cie=0+0060 pc=f+fffffc0..0+0000
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop
  DW_CFA_nop

